
import Pyro.naming
import Pyro.nsc
import sys
import os
import string
import signal
import subprocess
import coopr.opt

def coopr_ns():
    Pyro.naming.main(sys.argv[1:])

def coopr_nsc():
    Pyro.nsc.main(sys.argv[1:])

def kill_pyro_mip_servers():
    if len(sys.argv) > 2:
        print "***Incorrect invocation - use: kill_pyro_mip_servers pid-filename"
        sys.exit(1)

    pid_filename = "pyro_mip_servers.pids"
    if len(sys.argv) == 2:
        pid_filename = sys.argv[1]

    print "Killing pyro mip servers specified in file="+pid_filename

    pid_file = open(pid_filename, "r")
    for line in pid_file.readlines():
        pid = eval(string.strip(line))
        print "KILLING PID="+str(pid)
        os.kill(pid, signal.SIGTERM)
    pid_file.close()

def launch_pyro_mip_servers():
    if len(sys.argv) != 2:
        print "***Incorrect invocation - use: launch_pyro_mip_servers num-servers"
        sys.exit(1)

    num_servers = eval(sys.argv[1])

    print "Number of servers to launch="+str(num_servers)

    server_pids = []

    for i in range(1, num_servers+1):
        print "Launching server number "+str(i)
        output_filename = "pyro_mip_server"+str(i)+".out"
        # the "exec" ensures that (at least for bash) that the server process
        # will be the process returned, i.e., it becomes the child process - no
        # shell process intermediate. more correctly, exec exits the current
        # process before it does so (no fork).
        pid=subprocess.Popen("exec pyro_mip_server >& pyro_mip_server."+str(i)+".out", shell=True).pid
    #   print "PID="+str(pid)
        server_pids.append(pid)

    # perhaps a better place would be in the users home directory, but I'll
    # worry about that a bit later.
    pid_output_filename = "pyro_mip_servers.pids"
    pid_output_file = open(pid_output_filename,"w")
    for pid in server_pids:
        print >>pid_output_file, pid
    pid_output_file.close()

    print "PIDs for launched servers recorded in file="+pid_output_filename

def OSSolverService():
    if len(sys.argv) == 1:
        print "OSSolverService -osil <filename> -solver <name>"
        sys.exit(1)

    osilFile = None
    solver = None
    i=1
    while i<len(sys.argv):
        if sys.argv[i] == "-osil":
            i=i+1
            osilFile=sys.argv[i]
        elif sys.argv[i] == "-solver":
            i=i+1
            solver=sys.argv[i]
        i=i+1

    print "osilFile",osilFile,"solver",solver

    opt = coopr.opt.SolverFactory(solver)
    opt.solve(osilFile, rformat=coopr.opt.ResultsFormat.osrl)

def readsol():
    reader = coopr.opt.ReaderFactory("sol")
    soln = reader(sys.argv[1])
    soln.write()



#! /usr/bin/env python
#
# pyro_mip_server: A script that sets up a Pyro server for solving MIPs in
#           a distributed manner.
#
#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the FAST README.txt file.
#  _________________________________________________________________________

import os
import os.path
import sys
try:
    import cPickle as pickle
except ImportError:
    import pickle
import pyutilib.services
import pyutilib.pyro
import coopr.opt


class CooprMIPWorker(pyutilib.pyro.TaskWorker):

    def process(self, data):
        # construct the solver on this end, based on the input type stored in "data.opt".
        opt = coopr.opt.SolverFactory(data.opt)
        if opt is None:
            raise ValueError, "Problem constructing solver `"+data.opt+"'"
        opt.suffixes = data.suffixes

        # here is where we should set any options required by the solver, available
        # as specific attributes of the input data object.
        solver_options = data.solver_options
        del data.solver_options
        for key,value in solver_options.items():
#            print "RECEIVED SOLVER OPTION="+key+" WITH VALUE="+str(value)
            setattr(opt.options,key,value)

        problem_filename_suffix = os.path.split(data.filename)[1]
        temp_problem_filename = pyutilib.services.TempfileManager.create_tempfile(suffix="."+problem_filename_suffix)
        OUTPUT=open(temp_problem_filename,'w')
        print >>OUTPUT, data.file,
        OUTPUT.close()

        if data.warmstart_file is not None:
            warmstart_filename_suffix = os.path.split(data.warmstart_filename)[1]
            temp_warmstart_filename = pyutilib.services.TempfileManager.create_tempfile(suffix="."+warmstart_filename_suffix)
            OUTPUT=open(temp_warmstart_filename,'w')
            print >>OUTPUT, data.warmstart_file
            OUTPUT.close()
            opt.warm_start_solve = True
            opt.warm_start_file_name = temp_warmstart_filename

        print "Applying solver="+data.opt+" to solve problem="+temp_problem_filename
        results = opt.solve(temp_problem_filename, **data.kwds)
        # attach the symbol map.
        results.symbol_map = data.symbol_map
        pyutilib.services.TempfileManager.clear_tempfiles()
        # disabling write of solutions for now - this was getting too verbose,
        # especially when running Progressive Hedging.
        print "Solve completed - number of solutions="+str(len(results.solution))
        sys.stdout.flush()
#        results.write()
#        sys.stdout.flush()
        return pickle.dumps(results)


def main():
    pyutilib.pyro.TaskWorkerServer(CooprMIPWorker, argv=sys.argv)



import sys
import os
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(dirname(abspath(__file__))))+os.sep+"coopr")
sys.path.append(".."+os.sep+"..")

from distutils.core import setup
import py2exe

setup(console=['../../bin/pysos'])



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

import pyutilib.dev.runtests
import sys
import os.path
import optparse

def runCooprTests():
    parser = optparse.OptionParser(usage='test.coopr [options] <dirs>')

    parser.add_option('-d','--dir',
        action='store',
        dest='dir',
        default=None,
        help='Top-level source directory where the tests are applied.')
    parser.add_option('--cat','--category',
        action='append',
        dest='cats',
        default=[],
        help='Specify test categories.')
    parser.add_option('--all',
        action='store_true',
        dest='all_cats',
        default=False,
        help='All tests are executed.')
    parser.add_option('-v','--verbose',
        action='store_true',
        dest='verbose',
        default=False,
        help='Verbose output')
    parser.add_option('-o','--output',
        action='store',
        dest='output',
        default=None,
        help='Redirect output to a file')

    _options, args = parser.parse_args(sys.argv)

    if _options.output:
        outfile = os.path.abspath(_options.output)
    else:
        outfile = None
    if _options.dir is None:
        os.chdir( os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))) )
    else:
        os.chdir( _options.dir )

    print "Running tests in directory",os.getcwd()
    if _options.all_cats is True:
        _options.cats = []
    elif len(_options.cats) == 0:
        _options.cats = ['smoke']
    elif 'all' in _options.cats:
        _options.cats = []
    if len(_options.cats) > 0:
        os.environ['PYUTILIB_UNITTEST_CATEGORIES'] = ",".join(_options.cats)
        print " ... for test categories: "+ os.environ['PYUTILIB_UNITTEST_CATEGORIES']
    options=[]
    if _options.verbose:
        options.append('-v')
    if outfile:
        options.append('-o')
        options.append(outfile)
    if len(args) == 1:
        dirs=['coopr']
    else:
        dirs=[]
        for dir in args:
            if dir.startswith('-'):
                options.append(dir)
            if dir.startswith('coopr'):
                dirs.append(dir)
            else:
                dirs.append('coopr.'+dir)
        if len(dirs) == 0:
            dirs = ['coopr']

    pyutilib.dev.runtests.run('coopr', ['runtests']+options+['-p','coopr']+dirs)



#
# A script that registers Coopr COM servers
#
#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the FAST README.txt file.
#  _________________________________________________________________________

import sys
if sys.version_info[0:2] < (2,4):
    print ""
    print "ERROR: Pyomo requires Python 2.4 or newer"
    sys.exit(1)
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__))))
sys.path.append(".")
import os
import commands
import coopr.pyomo.main_script
import pyutilib.misc

# NEVER copy the following ID
# Use "print pythoncom.CreateGuid()" to make a new one.

class PyomoCOMInterface:
    _public_methods_ = [ 'XRun', 'Run' ]
    _reg_progid_ = "Coopr.Pyomo"
    _reg_clsid_ = "{A07BB601-AB93-4379-8D69-01C2B0F08601}"

    def Run(self, directory, modelfile, datascript, logfile):
        try:
            pyutilib.misc.setup_redirect(logfile)
            sys.argv.append("--summary")
            sys.argv.append("--log")
            sys.argv.append(directory+os.sep+modelfile)
            sys.argv.append(directory+os.sep+datascript)
            coopr.pyomo.main_script.run()
            pyutilib.misc.reset_redirect()
            return logfile
        except Exception, err:
            return str(err)
        except BaseException, err:
            return str(err)
        except:
            pass
        return "Unknown Error"

    def XRun(self, directory, modelfile, datascript):
        cmd = "pyomo "+directory+os.sep+modelfile+" "+directory+os.sep+datascript
        return cmd


if __name__=='__main__':
    print "Registering Coopr.Pyomo COM server..."
    import win32com.server.register
    win32com.server.register.UseCommandLine(PyomoCOMInterface)
